package zap

import (
	"go.uber.org/zap"

	"go.uber.org/zap/zapcore"
	"amuz.es/src/infra/logger/rotater"
	"amuz.es/src/infra/logger"
)

var (
	defaultOptions []zap.Option
	defaultWriter  logger.RotateSyncer
	nopCloser      = func() (err error) { return }
)

func init() {
	zap.RedirectStdLog(zap.L())
}

func replaceGlobalLogger(newOne *zap.Logger) {
	zap.ReplaceGlobals(newOne)
	zap.RedirectStdLog(newOne)
}

func Init(
	verbose bool,
	formatter zapcore.Encoder,
	mainLogName, logFilename, logDir string,
	maxSizeMb, maxBackup, maxDay int,
	logLevel zapcore.Level,
) *zap.SugaredLogger {
	level := zap.NewAtomicLevelAt(logLevel)
	defaultWriter = rotater.NewLogWriter(logFilename, maxSizeMb, maxBackup, maxDay, logDir)

	//encoderOption := getEncoderOption(verbose)
	//formatter := zapcore.NewConsoleEncoder(encoderOption)

	// reset log option slice
	defaultOptions = getOptions(defaultWriter, verbose)

	log := initLogger(defaultWriter, mainLogName, formatter, level, defaultOptions...)

	replaceGlobalLogger(log)
	return log.Sugar()
}

func New(parent *zap.SugaredLogger, moduleName string) *zap.SugaredLogger {
	if parent == nil {
		return zap.L().Named(moduleName).Sugar()
	} else {
		return parent.Named(moduleName)
	}
}

func NewOtherLogger(
	formatter zapcore.Encoder,
	moduleName, logFilename, logDir string,
	maxSizeMb, maxBackup, maxDay int,
	logLevel zapcore.Level,
	fields ...zapcore.Field,
) (logger *zap.SugaredLogger, closer func() error) {
	loglevel := zap.NewAtomicLevelAt(logLevel)
	logWriter := rotater.NewLogWriter(logFilename, maxSizeMb, maxBackup, maxDay, logDir)
	core := zapcore.NewCore(formatter, logWriter, loglevel)

	if rotateSyncer, ok := logWriter.(logger.RotateSyncer); ok {
		closer = rotateSyncer.Close
	} else {
		closer = nopCloser
	}

	options := []zap.Option{zap.ErrorOutput(defaultWriter)}
	logger = zap.New(core, options...).
		Named(moduleName).With(fields...).Sugar()
	return
}

func initLogger(
	writer zapcore.WriteSyncer,
	moduleName string,
	formatter zapcore.Encoder,
	level zap.AtomicLevel,
	options ...zap.Option,
) *zap.Logger {
	core := zapcore.NewCore(formatter, writer, level)
	return zap.New(core, options...).Named(moduleName)
}

func getOptions(errWriter zapcore.WriteSyncer, verbose bool) (options []zap.Option) {
	// define  error log
	options = append(options, zap.ErrorOutput(errWriter))
	if verbose {
		options = append(options, zap.AddStacktrace(zap.NewAtomicLevelAt(zap.PanicLevel)))
	}
	return
}
